<?php

namespace Drupal\avatar_field_formatter\Plugin\Field\FieldFormatter;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Transliteration\PhpTransliteration;
use Drupal\image\Plugin\Field\FieldFormatter\ImageFormatter;
use Laravolt\Avatar\Avatar;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the Avatar Field Formatter'.
 *
 * @FieldFormatter(
 *   id = "avatar_formatter_default",
 *   label = @Translation("Avatar default"),
 *   field_types = {
 *     "image"
 *   },
 *   quickedit = {
 *     "editor" = "image"
 *   }
 * )
 */
class AvatarFormatter extends ImageFormatter {

  /**
   * The transliteration manager.
   *
   * @var \Drupal\Core\Transliteration\PhpTransliteration
   */
  protected $transliteration;

  /**
   * The avatar library instance.
   *
   * @var \Laravolt\Avatar\Avatar
   */
  protected $avatar;

  /**
   * Constructs an ImageFormatter object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param \Drupal\Core\Entity\EntityStorageInterface $image_style_storage
   *   The image style storage.
   * @param \Drupal\Core\Transliteration\PhpTransliteration $transliteration
   *   The transliteration manager.
   * @param \Laravolt\Avatar\Avatar $avatar
   *   The avatar library instance.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, AccountInterface $current_user, EntityStorageInterface $image_style_storage, PhpTransliteration $transliteration, Avatar $avatar) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, $current_user, $image_style_storage);
    $this->transliteration = $transliteration;
    $this->avatar = $avatar;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $config = [
      'backgrounds'   => [
        '#f44336',
        '#E91E63',
        '#9C27B0',
        '#673AB7',
        '#3F51B5',
        '#2196F3',
        '#03A9F4',
        '#00BCD4',
        '#009688',
        '#4CAF50',
        '#8BC34A',
        '#CDDC39',
        '#FFC107',
        '#FF9800',
        '#FF5722',
      ],
      'chars' => 1,
      'uppercase' => TRUE,
    ];
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('current_user'),
      $container->get('entity.manager')->getStorage('image_style'),
      $container->get('transliteration'),
      new Avatar($config)
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'avatar_height' => '100',
      'avatar_width' => '100',
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {

    $element = parent::settingsForm($form, $form_state);

    $element['avatar_width'] = [
      '#title' => $this->t('Avatar Width (px)'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('avatar_width'),
      '#min' => 1,
    ];

    $element['avatar_height'] = [
      '#title' => $this->t('Avatar Height (px)'),
      '#type' => 'number',
      '#default_value' => $this->getSetting('avatar_height'),
      '#min' => 1,
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {

    $elements = parent::viewElements($items, $langcode);

    if (empty($elements)) {
      $width = $this->getSetting('avatar_width');
      $height = $this->getSetting('avatar_height');
      $display_name = $this->currentUser->getDisplayName();
      $avatar_letters = ucfirst($this->transliteration->transliterate($display_name));
      $data = $this->avatar->create($avatar_letters)
        ->setDimension($width, $height)
        ->toBase64();
      $elements[] =
        [
          '#type' => 'inline_template',
          '#template' => '<img src="{{ data }}" alt="{{ alt }}"/>',
          '#context' => [
            'data' => $data,
            'alt' => $display_name,
          ],
        ];
    }
    return $elements;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    $width = $this->getSetting('avatar_width');
    $height = $this->getSetting('avatar_height');
    $summary[] = $this->t('Avatar: @widthpx x @heightpx',
      [
        '@width' => $width,
        '@height' => $height,
      ]);

    return $summary;
  }

}
